@@ -24,6 +24,10 @@
  * size.
  *************************************************************************/
 
+#ifdef _MSC_VER
+#pragma warning(disable:4244)  // Conversion warnings
+#endif
+
 #include "mfcpch.h"
 #include          <stdlib.h>
 #include          <string.h>
