@@ -19,6 +19,7 @@
 
 #include          "mfcpch.h"     //precompiled headers
 #include          <stdlib.h>
+#include          "helpers.h"
 #include          "ndminx.h"
 #include          "serialis.h"
 #include          "points.h"
@@ -55,6 +56,24 @@ static int sign(int x) {
     return x > 0 ? 1 : 0;
 }
 
+// Writes to the given file. Returns false in case of error.
+bool ICOORD::Serialize(FILE* fp) const {
+  if (fwrite(&xcoord, sizeof(xcoord), 1, fp) != 1) return false;
+  if (fwrite(&ycoord, sizeof(ycoord), 1, fp) != 1) return false;
+  return true;
+}
+// Reads from the given file. Returns false in case of error.
+// If swap is true, assumes a big/little-endian swap is needed.
+bool ICOORD::DeSerialize(bool swap, FILE* fp) {
+  if (fread(&xcoord, sizeof(xcoord), 1, fp) != 1) return false;
+  if (fread(&ycoord, sizeof(ycoord), 1, fp) != 1) return false;
+  if (swap) {
+    ReverseN(&xcoord, sizeof(xcoord));
+    ReverseN(&ycoord, sizeof(ycoord));
+  }
+  return true;
+}
+
 // Setup for iterating over the pixels in a vector by the well-known
 // Bresenham rendering algorithm.
 // Starting with major/2 in the accumulator, on each step add major_step,
