@@ -24,7 +24,7 @@
 #endif
 #define FASTEDGELENGTH    256
 #include          "polyaprx.h"
-#include          "varable.h"
+#include          "params.h"
 #include          "tprintf.h"
 
 #define EXTERN
@@ -33,50 +33,33 @@ EXTERN BOOL_VAR (poly_debug, FALSE, "Debug old poly");
 EXTERN BOOL_VAR (poly_wide_objects_better, TRUE,
 "More accurate approx on wide things");
 
-static int par1, par2;
 
-#define CONVEX        1          /*OUTLINE point is convex */
-#define CONCAVE       2          /*used and set only in edges */
 #define FIXED       4            /*OUTLINE point is fixed */
-#define ONHULL        8          /*on convex hull */
 
 #define RUNLENGTH     1          /*length of run */
 
 #define DIR         2            /*direction of run */
 
-#define CORRECTION      3        /*correction of run */
-//#define MAXSHORT                      32767                                           /*max value of short*/
 #define FLAGS       0
 
 #define fixed_dist      20       //really an int_variable
 #define approx_dist     15       //really an int_variable
 
-#define point_diff(p,p1,p2) (p).x = (p1).x - (p2).x ; (p).y = (p1).y - (p2).y
-#define CROSS(a,b) ((a).x * (b).y - (a).y * (b).x)
-#define LENGTH(a) ((a).x * (a).x + (a).y * (a).y)
+const int par1 = 4500 / (approx_dist * approx_dist);
+const int par2 = 6750 / (approx_dist * approx_dist);
 
-#define DISTANCE(a,b) (((b).x-(a).x) * ((b).x-(a).x) \
-                        + ((b).y-(a).y) * ((b).y-(a).y))
 
 /**********************************************************************
  * tesspoly_outline
  *
- * Approximate an outline from c form using the old tess algorithm.
+ * Approximate an outline from chain codes form using the old tess algorithm.
  **********************************************************************/
 
-OUTLINE *tesspoly_outline(                       //old approximation
-                          C_OUTLINE *c_outline,  //input
-                          float                  //xheight
-                         ) {
-  EDGEPT *edgept;                //converted steps
-  EDGEPT *startpt;               //start of outline
-  TBOX loop_box;                  //bounding box
-  inT32 area;                    //loop area
-  FCOORD pos;                    //vertex
-  FCOORD vec;                    //vector
-  POLYPT_LIST polypts;           //output polygon
-  POLYPT *polypt;                //converted point
-  POLYPT_IT poly_it = &polypts;  //iterator
+
+TESSLINE* ApproximateOutline(C_OUTLINE* c_outline) {
+  EDGEPT *edgept;                // converted steps
+  TBOX loop_box;                  // bounding box
+  inT32 area;                    // loop area
   EDGEPT stack_edgepts[FASTEDGELENGTH];  // converted path
   EDGEPT* edgepts = stack_edgepts;
 
@@ -84,30 +67,36 @@ OUTLINE *tesspoly_outline(                       //old approximation
   if (c_outline->pathlength() > FASTEDGELENGTH)
     edgepts = new EDGEPT[c_outline->pathlength()];
 
-  loop_box = c_outline->bounding_box ();
-  area = loop_box.height ();
-  if (!poly_wide_objects_better && loop_box.width () > area)
-    area = loop_box.width ();
+  loop_box = c_outline->bounding_box();
+  area = loop_box.height();
+  if (!poly_wide_objects_better && loop_box.width() > area)
+    area = loop_box.width();
   area *= area;
-  edgept = edgesteps_to_edgepts (c_outline, edgepts);
+  edgept = edgesteps_to_edgepts(c_outline, edgepts);
   fix2(edgepts, area);
-  edgept = poly2 (edgepts, area);/*2nd approximation */
-  startpt = edgept;
+  edgept = poly2 (edgepts, area);  // 2nd approximation.
+  EDGEPT* startpt = edgept;
+  EDGEPT* result = NULL;
+  EDGEPT* prev_result = NULL;
   do {
-    pos = FCOORD (edgept->pos.x, edgept->pos.y);
-    vec = FCOORD (edgept->vec.x, edgept->vec.y);
-    polypt = new POLYPT (pos, vec);
-                                 //add to list
-    poly_it.add_after_then_move (polypt);
+    EDGEPT* new_pt = new EDGEPT;
+    new_pt->pos = edgept->pos;
+    new_pt->prev = prev_result;
+    if (prev_result == NULL) {
+      result = new_pt;
+    } else {
+      prev_result->next = new_pt;
+      new_pt->prev = prev_result;
+    }
+    prev_result = new_pt;
     edgept = edgept->next;
   }
   while (edgept != startpt);
+  prev_result->next = result;
+  result->prev = prev_result;
   if (edgepts != stack_edgepts)
     delete [] edgepts;
-  if (poly_it.length() <= 2)
-    return NULL;
-  else
-    return new OUTLINE(&poly_it);
+  return TESSLINE::BuildFromOutlineList(result);
 }
 
 
@@ -349,6 +338,11 @@ void fix2(                //polygonal approx
       break;                     //already too few
     point_diff (d12vec, edgefix1->pos, edgefix2->pos);
     d12 = LENGTH (d12vec);
+    // TODO(rays) investigate this change:
+    // Only unfix a point if it is part of a low-curvature section
+    // of outline and the total angle change of the outlines is
+    // less than 90 degrees, ie the scalar product is positive.
+    // if (d12 <= gapmin && SCALAR(edgefix0->vec, edgefix2->vec) > 0) {
     if (d12 <= gapmin) {
       point_diff (d01vec, edgefix0->pos, edgefix1->pos);
       d01 = LENGTH (d01vec);
@@ -357,16 +351,10 @@ void fix2(                //polygonal approx
       if (d01 > d23) {
         edgefix2->flags[FLAGS] &= ~FIXED;
         fixed_count--;
-        /*                  if ( plots[EDGE] & PATHS )
-                  mark(edgefd,edgefix2->pos.x,edgefix2->pos.y,PLUS);
-                                  */
       }
       else {
         edgefix1->flags[FLAGS] &= ~FIXED;
         fixed_count--;
-        /*                  if ( plots[EDGE] & PATHS )
-                  mark(edgefd,edgefix1->pos.x,edgefix1->pos.y,PLUS);
-                                    */
         edgefix1 = edgefix2;
       }
     }
@@ -407,11 +395,6 @@ EDGEPT *poly2(                  //second poly
   if (area < 1200)
     area = 1200;                 /*minimum value */
 
-                                 /*1200(4) */
-  par1 = 4500 / (approx_dist * approx_dist);
-                                 /*1200(6) */
-  par2 = 6750 / (approx_dist * approx_dist);
-
   loopstart = NULL;              /*not found it yet */
   edgept = startpt;              /*start of loop */
 
