@@ -14,10 +14,10 @@ CCUtil::CCUtil() :
 #endif
   INT_INIT_MEMBER(ambigs_debug_level, 0, "Debug level for unichar ambiguities",
                   &params_),
-  BOOL_INIT_MEMBER(use_definite_ambigs_for_classifier, 0, "Use definite"
-                   " ambiguities when running character classifier", &params_),
-  BOOL_INIT_MEMBER(use_ambigs_for_adaption, 0, "Use ambigs for deciding"
-                   " whether to adapt to a character", &params_) {
+  BOOL_MEMBER(use_definite_ambigs_for_classifier, 0, "Use definite"
+              " ambiguities when running character classifier", &params_),
+  BOOL_MEMBER(use_ambigs_for_adaption, 0, "Use ambigs for deciding"
+              " whether to adapt to a character", &params_) {
 }
 
 CCUtil::~CCUtil() {
