@@ -21,31 +21,6 @@
 #include <stdlib.h>
 #include "elst.h"
 
-/***********************************************************************
- *  MEMBER FUNCTIONS OF CLASS: ELIST_LINK
- *  =====================================
- **********************************************************************/
-
-/***********************************************************************
- *							ELIST_LINK::serialise_asc
- *
- *  Generates an error as it should never be called.
- **********************************************************************/
-
-void ELIST_LINK::serialise_asc(  //default serialise
-                               FILE *f) {
-  SERIALISE_LINKS.error ("ELIST_LINK::serialise_asc", ABORT,
-    "Don't call this, override!");
-}
-
-
-void ELIST_LINK::de_serialise_asc(  //default de_serialise
-                                  FILE *f) {
-  SERIALISE_LINKS.error ("ELIST_LINK::de_serialise_asc", ABORT,
-    "Don't call this, override!");
-}
-
-
 /***********************************************************************
  *  MEMBER FUNCTIONS OF CLASS: ELIST
  *  ================================
@@ -124,8 +99,8 @@ void ELIST::assign_to_sublist(                           //to this list
  *  Return count of elements on list
  **********************************************************************/
 
-inT32 ELIST::length() {  //count elements
-  ELIST_ITERATOR it(this);
+inT32 ELIST::length() const {  // count elements
+  ELIST_ITERATOR it(const_cast<ELIST*>(this));
   inT32 count = 0;
 
   #ifndef NDEBUG
@@ -190,8 +165,14 @@ const void *, const void *)) {
 // Comparision function is the same as used by sort, i.e. uses double
 // indirection. Time is O(1) to add to beginning or end.
 // Time is linear to add pre-sorted items to an empty list.
-void ELIST::add_sorted(int comparator(const void*, const void*),
-                       ELIST_LINK* new_link) {
+// If unique is set to true and comparator() returns 0 (an entry with the
+// same information as the one contained in new_link is already in the
+// list) - new_link is not added to the list and the function returns the
+// pointer to the identical entry that already exists in the list
+// (otherwise the function returns new_link).
+ELIST_LINK *ELIST::add_sorted_and_find(
+    int comparator(const void*, const void*),
+    bool unique, ELIST_LINK* new_link) {
   // Check for adding at the end.
   if (last == NULL || comparator(&last, &new_link) < 0) {
     if (last == NULL) {
@@ -206,101 +187,21 @@ void ELIST::add_sorted(int comparator(const void*, const void*),
     ELIST_ITERATOR it(this);
     for (it.mark_cycle_pt(); !it.cycled_list(); it.forward()) {
       ELIST_LINK* link = it.data();
-      if (comparator(&link, &new_link) > 0)
+      int compare = comparator(&link, &new_link);
+      if (compare > 0) {
         break;
+      } else if (unique && compare == 0) {
+        return link;
+      }
     }
     if (it.cycled_list())
       it.add_to_end(new_link);
     else
       it.add_before_then_move(new_link);
   }
+  return new_link;
 }
 
-/***********************************************************************
- *							ELIST::prep_serialise
- *
- *  Replace the last member with a count of elements for serialisation.
- *  This is used on list objects which are members of objects being
- *  serialised.  The containing object has been shallow copied and this member
- *  function is invoked on the COPY.
- **********************************************************************/
-
-void ELIST::prep_serialise() {
-  ELIST_ITERATOR this_it(this);
-  inT32 count = 0;
-
-  #ifndef NDEBUG
-  if (!this)
-    NULL_OBJECT.error ("ELIST::prep_serialise", ABORT, NULL);
-  #endif
-
-  count = 0;
-  if (!empty ())
-    for (this_it.mark_cycle_pt ();
-    !this_it.cycled_list (); this_it.forward ())
-  count++;
-  last = (ELIST_LINK *) count;
-}
-
-
-/***********************************************************************
- *							ELIST::internal_dump
- *
- *  Cause each element on the list to be serialised by walking the list and
- *  calling the element_serialiser function for each element.  The
- *  element_serialiser simply does the appropriate coercion of the element to
- *  its real type and then invokes the elements serialise function
- **********************************************************************/
-
-void
-ELIST::internal_dump (FILE * f,
-void element_serialiser (FILE *, ELIST_LINK *)) {
-  ELIST_ITERATOR this_it(this);
-
-  #ifndef NDEBUG
-  if (!this)
-    NULL_OBJECT.error ("ELIST::internal_dump", ABORT, NULL);
-  #endif
-
-  if (!empty ())
-    for (this_it.mark_cycle_pt ();
-    !this_it.cycled_list (); this_it.forward ())
-  element_serialiser (f, this_it.data ());
-}
-
-
-/***********************************************************************
- *							ELIST::internal_de_dump
- *
- *  Cause each element on the list to be de_serialised by extracting the count
- *  of elements on the list, (held in the last member of the dumped version of
- *  the list object), and then de-serialising that number of list elements,
- *  adding each to the end of the reconstructed list.
- **********************************************************************/
-
-void
-ELIST::internal_de_dump (FILE * f,
-ELIST_LINK * element_de_serialiser (FILE *)) {
-  inT32 count = (ptrdiff_t) last;
-  ELIST_ITERATOR this_it;
-  ELIST_LINK *de_serialised_element;
-
-  #ifndef NDEBUG
-  if (!this)
-    NULL_OBJECT.error ("ELIST::internal_de_dump", ABORT, NULL);
-  #endif
-
-  last = NULL;
-  this_it.set_to_list (this);
-  for (; count > 0; count--) {
-    de_serialised_element = element_de_serialiser (f);
-                                 //ignore old ptr
-    de_serialised_element->next = NULL;
-    this_it.add_to_end (de_serialised_element);
-  }
-}
-
-
 /***********************************************************************
  *  MEMBER FUNCTIONS OF CLASS: ELIST_ITERATOR
  *  =========================================
