@@ -100,8 +100,8 @@ void ELIST2::assign_to_sublist(                            //to this list
  *  Return count of elements on list
  **********************************************************************/
 
-inT32 ELIST2::length() {  //count elements
-  ELIST2_ITERATOR it(this);
+inT32 ELIST2::length() const {  // count elements
+  ELIST2_ITERATOR it(const_cast<ELIST2*>(this));
   inT32 count = 0;
 
   #ifndef NDEBUG
@@ -195,93 +195,6 @@ void ELIST2::add_sorted(int comparator(const void*, const void*),
   }
 }
 
-/***********************************************************************
- *							ELIST2::prep_serialise
- *
- *  Replace the last member with a count of elements for serialisation.
- *  This is used on list objects which are members of objects being
- *  serialised.  The containing object has been shallow copied and this member
- *  function is invoked on the COPY.
- **********************************************************************/
-
-void ELIST2::prep_serialise() {
-  ELIST2_ITERATOR this_it(this);
-  inT32 count = 0;
-
-  #ifndef NDEBUG
-  if (!this)
-    NULL_OBJECT.error ("ELIST2::prep_serialise", ABORT, NULL);
-  #endif
-
-  count = 0;
-  if (!empty ())
-    for (this_it.mark_cycle_pt ();
-    !this_it.cycled_list (); this_it.forward ())
-  count++;
-  last = (ELIST2_LINK *) count;
-}
-
-
-/***********************************************************************
- *							ELIST2::internal_dump
- *
- *  Cause each element on the list to be serialised by walking the list and
- *  calling the element_serialiser function for each element.  The
- *  element_serialiser simply does the appropriate coercion of the element to
- *  its real type and then invokes the elements serialise function
- **********************************************************************/
-
-void
-ELIST2::internal_dump (FILE * f,
-void element_serialiser (FILE *, ELIST2_LINK *)) {
-  ELIST2_ITERATOR this_it(this);
-
-  #ifndef NDEBUG
-  if (!this)
-    NULL_OBJECT.error ("ELIST2::internal_dump", ABORT, NULL);
-  #endif
-
-  if (!empty ())
-    for (this_it.mark_cycle_pt ();
-    !this_it.cycled_list (); this_it.forward ())
-  element_serialiser (f, this_it.data ());
-}
-
-
-/***********************************************************************
- *							ELIST2::internal_de_dump
- *
- *  Cause each element on the list to be de_serialised by extracting the count
- *  of elements on the list, (held in the last member of the dumped version of
- *  the list object), and then de-serialising that number of list elements,
- *  adding each to the end of the reconstructed list.
- **********************************************************************/
-
-void
-ELIST2::internal_de_dump (FILE * f,
-ELIST2_LINK * element_de_serialiser (FILE *)) {
-  inT32 count = (ptrdiff_t) last;
-  ELIST2_ITERATOR this_it;
-  ELIST2_LINK *de_serialised_element;
-
-  #ifndef NDEBUG
-  if (!this)
-    NULL_OBJECT.error ("ELIST2::internal_de_dump", ABORT, NULL);
-  #endif
-
-  last = NULL;
-  this_it.set_to_list (this);
-  for (; count > 0; count--) {
-    de_serialised_element = element_de_serialiser (f);
-                                 //ignore old ptr
-    de_serialised_element->next = NULL;
-                                 //ignore old ptr
-    de_serialised_element->prev = NULL;
-    this_it.add_to_end (de_serialised_element);
-  }
-}
-
-
 /***********************************************************************
  *  MEMBER FUNCTIONS OF CLASS: ELIST2_ITERATOR
  *  ==========================================
