@@ -27,44 +27,31 @@
 #endif
 #include          <stdlib.h>
 #include          "basedir.h"
-#include          "mainblk.h"
+#include          "ccutil.h"
 
 #define VARDIR        "configs/" /*variables files */
 #define EXTERN
 
-EXTERN DLLSYM STRING datadir;    //dir for data files
-                                 //name of image
-EXTERN DLLSYM STRING imagebasename;
-EXTERN BOOL_VAR (m_print_variables, FALSE,
-"Print initial values of all variables");
-EXTERN STRING_VAR (m_data_sub_dir, "tessdata/", "Directory for data files");
-/*
-EXTERN INT_VAR (memgrab_size, 0, "Preallocation size for batch use");*/
-
-
 const ERRCODE NO_PATH =
 "Warning:explicit path for executable will not be used for configs";
 static const ERRCODE USAGE = "Usage";
 
+namespace tesseract {
 /**********************************************************************
  * main_setup
  *
  * Main for mithras demo program. Read the arguments and set up globals.
  **********************************************************************/
 
-void main_setup(                 /*main demo program */
+void CCUtil::main_setup(                 /*main demo program */
                 const char *argv0,       //program name
-                const char *basename,    //name of image
-                int argc,                /*argument count */
-                const char *const *argv  /*arguments */
+                const char *basename     //name of image
                ) {
-  inT32 arg;                     /*argument */
-  inT32 offset;                  //for flag
-  FILE *fp;                      /*variables file */
-  char flag[2];                  //+/-
-  STRING varfile;                /*name of file */
-
   imagebasename = basename;      /*name of image */
+  STRING dll_module_name;
+  #ifdef __MSW32__
+  dll_module_name = tessedit_module_name;
+  #endif
 
   // TESSDATA_PREFIX Environment variable overrules everything.
   // Compiled in -DTESSDATA_PREFIX is next.
@@ -79,7 +66,7 @@ void main_setup(                 /*main demo program */
 #undef _STR
 #else
     if (argv0 != NULL) {
-      if (getpath(argv0, datadir) < 0)
+      if (getpath(argv0, dll_module_name, datadir) < 0)
 #ifdef __UNIX__
         CANTOPENFILE.error("main", ABORT, "%s to get path", argv0);
 #else
@@ -93,34 +80,6 @@ void main_setup(                 /*main demo program */
     datadir = getenv("TESSDATA_PREFIX");
   }
 
-  for (arg = 0; arg < argc; arg++) {
-    if (argv[arg][0] == '+' || argv[arg][0] == '-') {
-      offset = 1;
-      flag[0] = argv[arg][0];
-    }
-    else {
-      offset = 0;
-    }
-    flag[offset] = '\0';
-    varfile = flag;
-                                 /*attempt open */
-    fp = fopen (argv[arg] + offset, "r");
-    if (fp != NULL) {
-      fclose(fp);  /*was only to test */
-    }
-    else {
-      varfile += datadir;
-      varfile += m_data_sub_dir; /*data directory */
-      varfile += VARDIR;         /*variables dir */
-    }
-                                 /*actual name */
-    varfile += argv[arg] + offset;
-    read_variables_file (varfile.string ());
-  }
-
-  if (m_print_variables)
-    print_variables(stdout);  /*print them all */
-
-
   datadir += m_data_sub_dir;     /*data directory */
 }
+}  // namespace tesseract
