@@ -47,7 +47,6 @@ enum Ranks {
   RANK_LONG = 1,
   RANK_LONGLONG = 2,
   RANK_PTR      = INT_MAX // Special value used for pointers
-//  RANK_PTR      = 3 // Special value used for pointers
 };
 
 const enum Ranks kMinRank = RANK_CHAR;
@@ -214,56 +213,26 @@ double strtofloat(const char* s)
   return minus ? -f : f;
 }
 
-static int tess_vfscanf(FILE* stream, const char *format, va_list ap);
-
-int tess_fscanf(FILE* stream, const char *format, ...)
-{
-  va_list ap;
-  int rv;
-
-  va_start(ap, format);
-  rv = tess_vfscanf(stream, format, ap);
-  va_end(ap);
-
-  return rv;
-}
-
-#ifdef EMBEDDED
 int fscanf(FILE* stream, const char *format, ...)
 {
   va_list ap;
   int rv;
 
   va_start(ap, format);
-  rv = tess_vfscanf(stream, format, ap);
+  rv = vfscanf(stream, format, ap);
   va_end(ap);
 
   return rv;
 }
 
-int vfscanf(FILE* stream, const char *format, ...)
-{
-  va_list ap;
-  int rv;
-
-  va_start(ap, format);
-  rv = tess_vfscanf(stream, format, ap);
-  va_end(ap);
-
-  return rv;
-}
-#endif
-
-#ifndef _MSV_VER
-static
-int tess_vfscanf(FILE* stream, const char *format, va_list ap)
+int vfscanf(FILE* stream, const char *format, va_list ap)
 {
   const char *p = format;
   char ch;
   int q = 0;
   uintmax_t val = 0;
   int rank = RANK_INT;    // Default rank
-  unsigned int width = ~0;
+  unsigned int width = UINT_MAX;
   int base;
   int flags = 0;
   enum {
@@ -283,7 +252,6 @@ int tess_vfscanf(FILE* stream, const char *format, va_list ap)
   int matchinv = 0;   // Is match map inverted?
   unsigned char range_start = 0;
   off_t start_off = ftell(stream);
-  double fval;
 
   // Skip leading spaces
   SkipSpace(stream);
@@ -293,7 +261,7 @@ int tess_vfscanf(FILE* stream, const char *format, va_list ap)
       case ST_NORMAL:
         if (ch == '%') {
           state = ST_FLAGS;
-          flags = 0; rank = RANK_INT; width = ~0;
+          flags = 0; rank = RANK_INT; width = UINT_MAX;
         } else if (isspace(static_cast<unsigned char>(ch))) {
           SkipSpace(stream);
         } else {
@@ -445,7 +413,8 @@ int tess_vfscanf(FILE* stream, const char *format, va_list ap)
                 break;
               }
 
-              fval = streamtofloat(stream);
+              {
+              double fval = streamtofloat(stream);
               switch(rank) {
                 case RANK_INT:
                   *va_arg(ap, float *) = static_cast<float>(fval);
@@ -455,6 +424,7 @@ int tess_vfscanf(FILE* stream, const char *format, va_list ap)
                 break;
               }
               converted++;
+              }
             break;
 
             case 'c':               // Character
@@ -569,11 +539,8 @@ int tess_vfscanf(FILE* stream, const char *format, va_list ap)
 
   return converted;
 }
-#endif
 
-#ifdef EMBEDDED
 int creat(const char *pathname, mode_t mode)
 {
   return open(pathname, O_CREAT | O_TRUNC | O_WRONLY, mode);
 }
-#endif
