@@ -27,9 +27,6 @@
  ********************************************************************************
  */
 
-#ifdef __cplusplus
-#define EXTERN extern "C"
-#else
 #define EXTERN extern
-#endif
+
 #endif
