@@ -35,14 +35,44 @@
 #include "dawg.h"
 #include "dict.h"
 #include "freelist.h"
+#include "genericvector.h"
 #include "helpers.h"
 
 namespace tesseract {
 
+const char kDoNotReverse[] = "RRP_DO_NO_REVERSE";
+const char kReverseIfHasRTL[] = "RRP_REVERSE_IF_HAS_RTL";
+const char kForceReverse[] = "RRP_FORCE_REVERSE";
+
+const char * const RTLReversePolicyNames[] = {
+  kDoNotReverse,
+  kReverseIfHasRTL,
+  kForceReverse
+};
+
+const char Trie::kAlphaPatternUnicode[] = "\u2000";
+const char Trie::kDigitPatternUnicode[] = "\u2001";
+const char Trie::kAlphanumPatternUnicode[] = "\u2002";
+const char Trie::kPuncPatternUnicode[] = "\u2003";
+const char Trie::kLowerPatternUnicode[] = "\u2004";
+const char Trie::kUpperPatternUnicode[] = "\u2005";
+
+const char *Trie::get_reverse_policy_name(RTLReversePolicy reverse_policy) {
+  return RTLReversePolicyNames[reverse_policy];
+}
+
+// Reset the Trie to empty.
+void Trie::clear() {
+  nodes_.delete_data_pointers();
+  nodes_.clear();
+  num_edges_ = 0;
+  new_dawg_node();  // Need to allocate node 0.
+}
+
 bool Trie::edge_char_of(NODE_REF node_ref, NODE_REF next_node,
                         int direction, bool word_end, UNICHAR_ID unichar_id,
                         EDGE_RECORD **edge_ptr, EDGE_INDEX *edge_index) const {
-  if (dawg_debug_level == 3) {
+  if (debug_level_ == 3) {
     tprintf("edge_char_of() given node_ref " REFFORMAT " next_node " REFFORMAT
             " direction %d word_end %d unichar_id %d, exploring node:\n",
             node_ref, next_node, direction, word_end, unichar_id);
@@ -90,8 +120,9 @@ bool Trie::edge_char_of(NODE_REF node_ref, NODE_REF next_node,
   return false;  // not found
 }
 
-bool Trie::add_edge_linkage(NODE_REF node1, NODE_REF node2, int direction,
-                            bool word_end, UNICHAR_ID unichar_id) {
+bool Trie::add_edge_linkage(NODE_REF node1, NODE_REF node2, bool marker_flag,
+                            int direction, bool word_end,
+                            UNICHAR_ID unichar_id) {
   if (num_edges_ == max_num_edges_) return false;
   EDGE_VECTOR *vec = (direction == FORWARD_EDGE) ?
     &(nodes_[node1]->forward_edges) : &(nodes_[node1]->backward_edges);
@@ -107,13 +138,13 @@ bool Trie::add_edge_linkage(NODE_REF node1, NODE_REF node2, int direction,
     search_index = vec->size();  // add is unsorted, so index does not matter
   }
   EDGE_RECORD edge_rec;
-  link_edge(&edge_rec, node2, direction, word_end, unichar_id);
+  link_edge(&edge_rec, node2, marker_flag, direction, word_end, unichar_id);
   if (search_index < vec->size()) {
     vec->insert(edge_rec, search_index);
   } else {
     vec->push_back(edge_rec);
   }
-  if (dawg_debug_level > 1) {
+  if (debug_level_ > 1) {
     tprintf("new edge in nodes_[" REFFORMAT "]: ", node1);
     print_edge_rec(edge_rec);
     tprintf("\n");
@@ -124,22 +155,35 @@ bool Trie::add_edge_linkage(NODE_REF node1, NODE_REF node2, int direction,
 
 void Trie::add_word_ending(EDGE_RECORD *edge_ptr,
                            NODE_REF the_next_node,
+                           bool marker_flag,
                            UNICHAR_ID unichar_id) {
   EDGE_RECORD *back_edge_ptr;
   EDGE_INDEX back_edge_index;
   ASSERT_HOST(edge_char_of(the_next_node, NO_EDGE, BACKWARD_EDGE, false,
                            unichar_id, &back_edge_ptr, &back_edge_index));
+  if (marker_flag) {
+    *back_edge_ptr |= (MARKER_FLAG << flag_start_bit_);
+    *edge_ptr |= (MARKER_FLAG << flag_start_bit_);
+  }
   // Mark both directions as end of word.
   *back_edge_ptr |= (WERD_END_FLAG << flag_start_bit_);
   *edge_ptr |= (WERD_END_FLAG << flag_start_bit_);
 }
 
-void Trie::add_word_to_dawg(const WERD_CHOICE &word) {
-  if (word.length() <= 0) return;  // can't add empty words
+bool Trie::add_word_to_dawg(const WERD_CHOICE &word,
+                            const GenericVector<bool> *repetitions) {
+  if (word.length() <= 0) return false;  // can't add empty words
+  if (repetitions != NULL) ASSERT_HOST(repetitions->size() == word.length());
+  // Make sure the word does not contain invalid unchar ids.
+  for (int i = 0; i < word.length(); ++i) {
+    if (word.unichar_id(i) < 0 ||
+        word.unichar_id(i) >= unicharset_size_) return false;
+  }
 
   EDGE_RECORD *edge_ptr;
   NODE_REF last_node = 0;
   NODE_REF the_next_node;
+  bool marker_flag = false;
   EDGE_INDEX edge_index;
   int i;
   inT32 still_finding_chars = true;
@@ -147,16 +191,17 @@ void Trie::add_word_to_dawg(const WERD_CHOICE &word) {
   bool  add_failed = false;
   bool found;
 
-  if (dawg_debug_level > 1) word.print("\nAdding word: ");
+  if (debug_level_ > 1) word.print("\nAdding word: ");
 
   UNICHAR_ID unichar_id;
   for (i = 0; i < word.length() - 1; ++i) {
     unichar_id = word.unichar_id(i);
-    if (dawg_debug_level > 1) tprintf("Adding letter %d\n", unichar_id);
+    marker_flag = (repetitions != NULL) ? (*repetitions)[i] : false;
+    if (debug_level_ > 1) tprintf("Adding letter %d\n", unichar_id);
     if (still_finding_chars) {
       found = edge_char_of(last_node, NO_EDGE, FORWARD_EDGE, word_end,
                            unichar_id, &edge_ptr, &edge_index);
-      if (found && dawg_debug_level > 1) {
+      if (found && debug_level_ > 1) {
         tprintf("exploring edge " REFFORMAT " in node " REFFORMAT "\n",
                 edge_index, last_node);
       }
@@ -167,18 +212,20 @@ void Trie::add_word_to_dawg(const WERD_CHOICE &word) {
         still_finding_chars = false;
         remove_edge(last_node, 0, word_end, unichar_id);
       } else {
+        if (marker_flag) set_marker_flag_in_edge_rec(edge_ptr);
         last_node = next_node_from_edge_rec(*edge_ptr);
       }
     }
     if (!still_finding_chars) {
       the_next_node = new_dawg_node();
-      if (dawg_debug_level > 1)
+      if (debug_level_ > 1)
         tprintf("adding node " REFFORMAT "\n", the_next_node);
       if (the_next_node == 0) {
         add_failed = true;
         break;
       }
-      if (!add_new_edge(last_node, the_next_node, word_end, unichar_id)) {
+      if (!add_new_edge(last_node, the_next_node,
+                        marker_flag, word_end, unichar_id)) {
         add_failed = true;
         break;
       }
@@ -188,23 +235,26 @@ void Trie::add_word_to_dawg(const WERD_CHOICE &word) {
   }
   the_next_node = 0;
   unichar_id = word.unichar_id(i);
-  if (dawg_debug_level > 1) tprintf("Adding letter %d\n", unichar_id);
+  marker_flag = (repetitions != NULL) ? (*repetitions)[i] : false;
+  if (debug_level_ > 1) tprintf("Adding letter %d\n", unichar_id);
   if (still_finding_chars &&
       edge_char_of(last_node, NO_EDGE, FORWARD_EDGE, false,
                    unichar_id, &edge_ptr, &edge_index)) {
     // An extension of this word already exists in the trie, so we
     // only have to add the ending flags in both directions.
-    add_word_ending(edge_ptr, next_node_from_edge_rec(*edge_ptr), unichar_id);
+    add_word_ending(edge_ptr, next_node_from_edge_rec(*edge_ptr),
+                    marker_flag, unichar_id);
   } else {
     if (!add_failed &&
-        !add_new_edge(last_node, the_next_node, true, unichar_id))
+        !add_new_edge(last_node, the_next_node, marker_flag, true, unichar_id))
       add_failed = true;
   }
   if (add_failed) {
     tprintf("Re-initializing document dictionary...\n");
-    nodes_.delete_data_pointers();
-    num_edges_ = 0;
-    new_dawg_node();  // need to allocate node 0
+    clear();
+    return false;
+  } else {
+    return true;
   }
 }
 
@@ -216,9 +266,10 @@ NODE_REF Trie::new_dawg_node() {
 }
 
 bool Trie::read_word_list(const char *filename,
-                          const UNICHARSET &unicharset) {
+                          const UNICHARSET &unicharset,
+                          Trie::RTLReversePolicy reverse_policy) {
   FILE *word_file;
-  char string [CHARS_PER_LINE];
+  char string[CHARS_PER_LINE];
   int  word_count = 0;
 
   word_file = open_file (filename, "r");
@@ -226,8 +277,13 @@ bool Trie::read_word_list(const char *filename,
   while (fgets(string, CHARS_PER_LINE, word_file) != NULL) {
     chomp_string(string);  // remove newline
     WERD_CHOICE word(string, unicharset);
+    if ((reverse_policy == RRP_REVERSE_IF_HAS_RTL &&
+        word.has_rtl_unichar_id()) ||
+        reverse_policy == RRP_FORCE_REVERSE) {
+      word.reverse_and_mirror_unichar_ids();
+    }
     ++word_count;
-    if (dawg_debug_level && word_count % 10000 == 0)
+    if (debug_level_ && word_count % 10000 == 0)
       tprintf("Read %d words so far\n", word_count);
     if (word.length() != 0 && !word.contains_unichar_id(INVALID_UNICHAR_ID)) {
       if (!this->word_in_dawg(word)) {
@@ -237,24 +293,163 @@ bool Trie::read_word_list(const char *filename,
           return false;
         }
       }
-    } else if (dawg_debug_level) {
+    } else if (debug_level_) {
       tprintf("Skipping invalid word %s\n", string);
-      if (dawg_debug_level >= 3) word.print();
+      if (debug_level_ >= 3) word.print();
     }
   }
-  if (dawg_debug_level)
+  if (debug_level_)
     tprintf("Read %d words total.\n", word_count);
   fclose(word_file);
   return true;
 }
 
+void Trie::initialize_patterns(UNICHARSET *unicharset) {
+  unicharset->unichar_insert(kAlphaPatternUnicode);
+  alpha_pattern_ = unicharset->unichar_to_id(kAlphaPatternUnicode);
+  unicharset->unichar_insert(kDigitPatternUnicode);
+  digit_pattern_ = unicharset->unichar_to_id(kDigitPatternUnicode);
+  unicharset->unichar_insert(kAlphanumPatternUnicode);
+  alphanum_pattern_ = unicharset->unichar_to_id(kAlphanumPatternUnicode);
+  unicharset->unichar_insert(kPuncPatternUnicode);
+  punc_pattern_ = unicharset->unichar_to_id(kPuncPatternUnicode);
+  unicharset->unichar_insert(kLowerPatternUnicode);
+  lower_pattern_ = unicharset->unichar_to_id(kLowerPatternUnicode);
+  unicharset->unichar_insert(kUpperPatternUnicode);
+  upper_pattern_ = unicharset->unichar_to_id(kUpperPatternUnicode);
+  initialized_patterns_ = true;
+  unicharset_size_ = unicharset->size();
+}
+
+void Trie::unichar_id_to_patterns(UNICHAR_ID unichar_id,
+                                  const UNICHARSET &unicharset,
+                                  GenericVector<UNICHAR_ID> *vec) const {
+  bool is_alpha = unicharset.get_isalpha(unichar_id);
+  if (is_alpha) {
+    vec->push_back(alpha_pattern_);
+    vec->push_back(alphanum_pattern_);
+    if (unicharset.get_islower(unichar_id)) {
+      vec->push_back(lower_pattern_);
+    } else if (unicharset.get_isupper(unichar_id)) {
+      vec->push_back(upper_pattern_);
+    }
+  }
+  if (unicharset.get_isdigit(unichar_id)) {
+    vec->push_back(digit_pattern_);
+    if (!is_alpha) vec->push_back(alphanum_pattern_);
+  }
+  if (unicharset.get_ispunctuation(unichar_id)) {
+    vec->push_back(punc_pattern_);
+  }
+}
+
+UNICHAR_ID Trie::character_class_to_pattern(char ch) {
+  if (ch == 'c') {
+    return alpha_pattern_;
+  } else if (ch == 'd') {
+    return digit_pattern_;
+  } else if (ch == 'n') {
+    return alphanum_pattern_;
+  } else if (ch == 'p') {
+    return punc_pattern_;
+  } else if (ch == 'a') {
+    return lower_pattern_;
+  } else if (ch == 'A') {
+    return upper_pattern_;
+  } else {
+    return INVALID_UNICHAR_ID;
+  }
+}
+
+bool Trie::read_pattern_list(const char *filename,
+                             const UNICHARSET &unicharset) {
+  if (!initialized_patterns_) {
+    tprintf("please call initialize_patterns() before read_pattern_list()\n");
+    return false;
+  }
+
+  FILE *pattern_file = open_file (filename, "r");
+  if (pattern_file == NULL) {
+    tprintf("Error opening pattern file %s\n", filename);
+    return false;
+  }
+
+  int pattern_count = 0;
+  char string[CHARS_PER_LINE];
+  while (fgets(string, CHARS_PER_LINE, pattern_file) != NULL) {
+    chomp_string(string);  // remove newline
+    // Parse the pattern and construct a unichar id vector.
+    // Record the number of repetitions of each unichar in the parallel vector.
+    WERD_CHOICE word(&unicharset);
+    GenericVector<bool> repetitions_vec;
+    const char *str_ptr = string;
+    int step = unicharset.step(str_ptr);
+    bool failed = false;
+    while (step > 0) {
+      UNICHAR_ID curr_unichar_id = INVALID_UNICHAR_ID;
+      if (step == 1 && *str_ptr == '\\') {
+        ++str_ptr;
+        if (*str_ptr == '\\') {  // regular '\' unichar that was escaped
+          curr_unichar_id = unicharset.unichar_to_id(str_ptr, step);
+        } else {
+          if (word.length() < kSaneNumConcreteChars) {
+            tprintf("Please provide at least %d concrete characters at the"
+                    " beginning of the pattern\n", kSaneNumConcreteChars);
+            failed = true;
+            break;
+          }
+          // Parse character class from expression.
+          curr_unichar_id = character_class_to_pattern(*str_ptr);
+        }
+      } else {
+        curr_unichar_id = unicharset.unichar_to_id(str_ptr, step);
+      }
+      if (curr_unichar_id ==  INVALID_UNICHAR_ID) {
+        failed = true;
+        break;  // failed to parse this pattern
+      }
+      word.append_unichar_id(curr_unichar_id, 1, 0.0, 0.0);
+      repetitions_vec.push_back(false);
+      str_ptr += step;
+      step = unicharset.step(str_ptr);
+      // Check if there is a repetition pattern specified after this unichar.
+      if (step == 1 && *str_ptr == '\\' && *(str_ptr+1) == '*') {
+        repetitions_vec[repetitions_vec.size()-1] = true;
+        str_ptr += 2;
+        step = unicharset.step(str_ptr);
+      }
+    }
+    if (failed) {
+      tprintf("Invalid user pattern %s\n", string);
+      continue;
+    }
+    // Insert the pattern into the trie.
+    if (debug_level_ > 2) {
+      tprintf("Inserting expanded user pattern %s\n",
+              word.debug_string().string());
+    }
+    if (!this->word_in_dawg(word)) {
+      this->add_word_to_dawg(word, &repetitions_vec);
+      if (!this->word_in_dawg(word)) {
+        tprintf("Error: failed to insert pattern '%s'\n", string);
+      }
+    }
+    ++pattern_count;
+  }
+  if (debug_level_) {
+    tprintf("Read %d valid patterns from %s\n", pattern_count, filename);
+  }
+  fclose(pattern_file);
+  return true;
+}
+
 void Trie::remove_edge_linkage(NODE_REF node1, NODE_REF node2, int direction,
                                bool word_end, UNICHAR_ID unichar_id) {
-  EDGE_RECORD *edge_ptr;
-  EDGE_INDEX edge_index;
+  EDGE_RECORD *edge_ptr = NULL;
+  EDGE_INDEX edge_index = 0;
   ASSERT_HOST(edge_char_of(node1, node2, direction, word_end,
                            unichar_id, &edge_ptr, &edge_index));
-  if (dawg_debug_level > 1) {
+  if (debug_level_ > 1) {
     tprintf("removed edge in nodes_[" REFFORMAT "]: ", node1);
     print_edge_rec(*edge_ptr);
     tprintf("\n");
@@ -268,7 +463,7 @@ void Trie::remove_edge_linkage(NODE_REF node1, NODE_REF node2, int direction,
 }
 
 SquishedDawg *Trie::trie_to_dawg() {
-  if (dawg_debug_level > 2) {
+  if (debug_level_ > 2) {
     print_all("Before reduction:", MAX_NODE_EDGES_DISPLAY);
   }
   NODE_MARKER reduced_nodes = new bool[nodes_.size()];
@@ -276,7 +471,7 @@ SquishedDawg *Trie::trie_to_dawg() {
   this->reduce_node_input(0, reduced_nodes);
   delete[] reduced_nodes;
 
-  if (dawg_debug_level > 2) {
+  if (debug_level_ > 2) {
     print_all("After reduction:", MAX_NODE_EDGES_DISPLAY);
   }
   // Build a translation map from node indices in nodes_ vector to
@@ -302,22 +497,22 @@ SquishedDawg *Trie::trie_to_dawg() {
       NODE_REF node_ref = next_node_from_edge_rec(edge_rec);
       ASSERT_HOST(node_ref < nodes_.size());
       UNICHAR_ID unichar_id = unichar_id_from_edge_rec(edge_rec);
-      link_edge(edge_array_ptr, node_ref_map[node_ref], FORWARD_EDGE,
+      link_edge(edge_array_ptr, node_ref_map[node_ref], false, FORWARD_EDGE,
                 end_of_word_from_edge_rec(edge_rec), unichar_id);
-      if (j == end - 1) set_last_flag_in_edge_rec(edge_array_ptr);
+      if (j == end - 1) set_marker_flag_in_edge_rec(edge_array_ptr);
       ++edge_array_ptr;
     }
   }
   delete[] node_ref_map;
 
-  return new SquishedDawg(edge_array, num_forward_edges,
-                          type_, lang_, perm_, unicharset_size_);
+  return new SquishedDawg(edge_array, num_forward_edges, type_, lang_,
+                          perm_, unicharset_size_, debug_level_);
 }
 
 bool Trie::eliminate_redundant_edges(NODE_REF node,
                                      const EDGE_RECORD &edge1,
                                      const EDGE_RECORD &edge2) {
-  if (dawg_debug_level > 1) {
+  if (debug_level_ > 1) {
     tprintf("\nCollapsing node %d:\n", node);
     print_node(node, MAX_NODE_EDGES_DISPLAY);
     tprintf("Candidate edges: ");
@@ -330,7 +525,7 @@ bool Trie::eliminate_redundant_edges(NODE_REF node,
   NODE_REF next_node2 = next_node_from_edge_rec(edge2);
   TRIE_NODE_RECORD *next_node2_ptr = nodes_[next_node2];
   // Translate all edges going to/from next_node2 to go to/from next_node1.
-  EDGE_RECORD *edge_ptr;
+  EDGE_RECORD *edge_ptr = NULL;
   EDGE_INDEX edge_index;
   int i;
   // Remove the backward link in node to next_node2.
@@ -344,7 +539,8 @@ bool Trie::eliminate_redundant_edges(NODE_REF node,
     NODE_REF curr_next_node = next_node_from_edge_rec(bkw_edge);
     UNICHAR_ID curr_unichar_id = unichar_id_from_edge_rec(bkw_edge);
     int curr_word_end = end_of_word_from_edge_rec(bkw_edge);
-    add_edge_linkage(next_node1, curr_next_node, BACKWARD_EDGE,
+    bool marker_flag = marker_flag_from_edge_rec(bkw_edge);
+    add_edge_linkage(next_node1, curr_next_node, marker_flag, BACKWARD_EDGE,
                      curr_word_end, curr_unichar_id);
     // Relocate the corresponding forward edge in curr_next_node
     ASSERT_HOST(edge_char_of(curr_next_node, next_node2, FORWARD_EDGE,
@@ -354,7 +550,7 @@ bool Trie::eliminate_redundant_edges(NODE_REF node,
   }
   int next_node2_num_edges = (next_node2_ptr->forward_edges.size() +
                               next_node2_ptr->backward_edges.size());
-  if (dawg_debug_level > 1) {
+  if (debug_level_ > 1) {
     tprintf("removed %d edges from node " REFFORMAT "\n",
             next_node2_num_edges, next_node2);
   }
@@ -369,7 +565,7 @@ bool Trie::reduce_lettered_edges(EDGE_INDEX edge_index,
                                  NODE_REF node,
                                  const EDGE_VECTOR &backward_edges,
                                  NODE_MARKER reduced_nodes) {
-  if (dawg_debug_level > 1)
+  if (debug_level_ > 1)
     tprintf("reduce_lettered_edges(edge=" REFFORMAT ")\n", edge_index);
   // Compare each of the edge pairs with the given unichar_id.
   bool did_something = false;
@@ -418,7 +614,7 @@ void Trie::sort_edges(EDGE_VECTOR *edges) {
 
 void Trie::reduce_node_input(NODE_REF node,
                              NODE_MARKER reduced_nodes) {
-  if (dawg_debug_level > 1) {
+  if (debug_level_ > 1) {
     tprintf("reduce_node_input(node=" REFFORMAT ")\n", node);
     print_node(node, MAX_NODE_EDGES_DISPLAY);
   }
@@ -436,7 +632,7 @@ void Trie::reduce_node_input(NODE_REF node,
   }
   reduced_nodes[node] = true;  // mark as reduced
 
-  if (dawg_debug_level > 1) {
+  if (debug_level_ > 1) {
     tprintf("Node " REFFORMAT " after reduction:\n", node);
     print_node(node, MAX_NODE_EDGES_DISPLAY);
   }
