@@ -31,6 +31,8 @@
 
 #ifndef TESSERACT_VIEWER_SCROLLVIEW_H__
 #define TESSERACT_VIEWER_SCROLLVIEW_H__
+// TODO(rays) Move ScrollView into the tesseract namespace.
+#ifndef OCR_SCROLLVIEW_H__
 
 #include <stdio.h>
 
@@ -201,10 +203,8 @@ class ScrollView {
 * constructor, so this is not listed here)
 *******************************************************************************/
 
-#ifdef HAVE_LIBLEPT
 // Draw a Pix on (x,y).
   void Image(struct Pix* image, int x_pos, int y_pos);
-#endif
 
 // Flush buffers and update display.
   static void Update();
@@ -347,14 +347,12 @@ class ScrollView {
   int TranslateYCoordinate(int y);
 
  private:
-#ifdef HAVE_LIBLEPT
 // Transfers a binary Image.
   void TransferBinaryImage(struct Pix* image);
 // Transfers a gray scale Image.
   void TransferGrayImage(struct Pix* image);
 // Transfers a 32-Bit Image.
   void Transfer32bppImage(struct Pix* image);
-#endif
 
 // Sets up ScrollView, depending on the variables from the constructor.
   void Initialize(const char* name, int x_pos, int y_pos, int x_size,
@@ -392,10 +390,15 @@ class ScrollView {
   SVPolyLineBuffer* points_;
   // Whether the axis is reversed.
   bool y_axis_is_reversed_;
+  // Set to true only after the event handler has terminated.
+  bool event_handler_ended_;
   // If the y axis is reversed, flip all y values by ySize.
   int y_size_;
   // # of created windows (used to assign an id to each ScrollView* for svmap).
   static int nr_created_windows_;
+  // Serial number of sent images to ensure that the viewer knows they
+  // are distinct.
+  static int image_index_;
 
   // The stream through which the c++ client is connected to the server.
   static SVNetwork* stream_;
@@ -411,4 +414,5 @@ class ScrollView {
 #endif  // GRAPHICS_DISABLED
 };
 
+#endif  // OCR_SCROLLVIEW_H__
 #endif  // TESSERACT_VIEWER_SCROLLVIEW_H__
