@@ -29,7 +29,6 @@
 #include "closed.h"
 #include "cutil.h"
 #include "callcpp.h"
-//#include <process.h>
 #ifdef __UNIX__
 #include <assert.h>
 #endif
@@ -38,18 +37,17 @@
               V a r i a b l e s
 ----------------------------------------------------------------------*/
 #define TABLE_SIZE 2000
-HASH_TABLE global_hash = NULL;
 
 /*----------------------------------------------------------------------
               F u n c t i o n s
 ----------------------------------------------------------------------*/
-/**********************************************************************
- * hash_add
+/**
+ * @name hash_add
  *
  * Look in the hash table for a particular value. If it is not there
  * then add it.
- **********************************************************************/
-int hash_add(HASH_TABLE state_table, STATE *state) { 
+ */
+int hash_add(HASH_TABLE state_table, STATE *state) {
   int x;
   int i = 0;
   int table_limit = TABLE_SIZE;
@@ -73,19 +71,19 @@ int hash_add(HASH_TABLE state_table, STATE *state) {
       x = 0;
   }
   cprintf("warning: hash table is full");
-  
-  abort(); 
+
+  abort();
   return 0;
 }
 
 
-/**********************************************************************
- * hash_lookup
+/**
+ * @name hash_lookup
  *
  * Look in the hash table for a particular value. If the value is there
  * then return TRUE, FALSE otherwise.
- **********************************************************************/
-int hash_lookup(HASH_TABLE state_table, STATE *state) { 
+ */
+int hash_lookup(HASH_TABLE state_table, STATE *state) {
   int x;
   int i = 0;
   int table_limit = TABLE_SIZE;
@@ -109,25 +107,21 @@ int hash_lookup(HASH_TABLE state_table, STATE *state) {
   }
   cprintf ("warning: fell off end of hash table  (%x) %x\n",
     state->part2, state->part2 % table_limit);
-  abort(); 
+  abort();
   return 0;
 }
 
 
-/**********************************************************************
- * new_hash_table
+/**
+ * @name new_hash_table
  *
  * Create and initialize a hash table.
- **********************************************************************/
-HASH_TABLE new_hash_table() { 
+ */
+HASH_TABLE new_hash_table() {
   HASH_TABLE ht;
   int x;
 
-  if (global_hash == NULL)
-    ht = (HASH_TABLE) memalloc (TABLE_SIZE * sizeof (STATE));
-  else
-    ht = global_hash;
-
+  ht = (HASH_TABLE) memalloc (TABLE_SIZE * sizeof (STATE));
   for (x = 0; x < TABLE_SIZE; x++) {
     ht[x].part1 = NO_STATE;
     ht[x].part2 = NO_STATE;
