@@ -26,9 +26,10 @@
               I n c l u d e s
 ----------------------------------------------------------------------*/
 #include "gradechop.h"
-#include "debug.h"
+#include "wordrec.h"
 #include "olutil.h"
 #include "chop.h"
+#include "ndminx.h"
 #include <math.h>
 
 /*----------------------------------------------------------------------
@@ -51,13 +52,15 @@
 																		\
 	this_point = point1;                                 \
 	do {                                                 \
-		x_min = min (this_point->pos.x, x_min);           \
-		x_max = max (this_point->pos.x, x_max);           \
+		x_min = MIN (this_point->pos.x, x_min);           \
+		x_max = MAX (this_point->pos.x, x_max);           \
 		this_point = this_point->next;                    \
 	}                                                    \
 	while (this_point != point2 && this_point != point1) \
 
 
+namespace tesseract {
+
 /*----------------------------------------------------------------------
               F u n c t i o n s
 ----------------------------------------------------------------------*/
@@ -68,12 +71,12 @@
  * Part of the priority has already been calculated so just return the
  * additional amount for the bounding box type information.
  **********************************************************************/
-PRIORITY full_split_priority(SPLIT *split, inT16 xmin, inT16 xmax) {
+PRIORITY Wordrec::full_split_priority(SPLIT *split, inT16 xmin, inT16 xmax) {
   BOUNDS_RECT rect;
 
   set_outline_bounds (split->point1, split->point2, rect);
 
-  if (xmin < min (rect[0], rect[2]) && xmax > max (rect[1], rect[3]))
+  if (xmin < MIN (rect[0], rect[2]) && xmax > MAX (rect[1], rect[3]))
     return (999.0);
 
   return (grade_overlap (rect) +
@@ -89,16 +92,16 @@ PRIORITY full_split_priority(SPLIT *split, inT16 xmin, inT16 xmax) {
  *   0    =  "perfect"
  *   100  =  "no way jay"
  **********************************************************************/
-PRIORITY grade_center_of_blob(register BOUNDS_RECT rect) {
+PRIORITY Wordrec::grade_center_of_blob(register BOUNDS_RECT rect) {
   register PRIORITY grade;
 
   grade = (rect[1] - rect[0]) - (rect[3] - rect[2]);
   if (grade < 0)
     grade = -grade;
 
-  grade *= center_knob;
-  grade = min (CENTER_GRADE_CAP, grade);
-  return (max (0.0, grade));
+  grade *= chop_center_knob;
+  grade = MIN (CENTER_GRADE_CAP, grade);
+  return (MAX (0.0, grade));
 }
 
 
@@ -109,7 +112,7 @@ PRIORITY grade_center_of_blob(register BOUNDS_RECT rect) {
  *   0    =  "perfect"
  *   100  =  "no way jay"
  **********************************************************************/
-PRIORITY grade_overlap(register BOUNDS_RECT rect) {
+PRIORITY Wordrec::grade_overlap(register BOUNDS_RECT rect) {
   register PRIORITY grade;
   register inT16 width1;
   register inT16 width2;
@@ -118,17 +121,17 @@ PRIORITY grade_overlap(register BOUNDS_RECT rect) {
   width1 = rect[3] - rect[2];
   width2 = rect[1] - rect[0];
 
-  overlap = min (rect[1], rect[3]) - max (rect[0], rect[2]);
-  width1 = min (width1, width2);
+  overlap = MIN (rect[1], rect[3]) - MAX (rect[0], rect[2]);
+  width1 = MIN (width1, width2);
   if (overlap == width1)
     return (100.0);              /* Total overlap */
 
   width1 = 2 * overlap - width1; /* Extra penalty for too */
-  overlap += max (0, width1);    /* much overlap */
+  overlap += MAX (0, width1);    /* much overlap */
 
-  grade = overlap * overlap_knob;
+  grade = overlap * chop_overlap_knob;
 
-  return (max (0.0, grade));
+  return (MAX (0.0, grade));
 }
 
 
@@ -139,19 +142,19 @@ PRIORITY grade_overlap(register BOUNDS_RECT rect) {
  *   0    =  "perfect"
  *   100  =  "no way jay"
  **********************************************************************/
-PRIORITY grade_split_length(register SPLIT *split) {
+PRIORITY Wordrec::grade_split_length(register SPLIT *split) {
   register PRIORITY grade;
   register float split_length;
 
   split_length = weighted_edgept_dist (split->point1, split->point2,
-    x_y_weight);
+    chop_x_y_weight);
 
   if (split_length <= 0)
     grade = 0;
   else
-    grade = sqrt (split_length) * split_dist_knob;
+    grade = sqrt (split_length) * chop_split_dist_knob;
 
-  return (max (0.0, grade));
+  return (MAX (0.0, grade));
 }
 
 
@@ -162,7 +165,7 @@ PRIORITY grade_split_length(register SPLIT *split) {
  *   0    =  "perfect"
  *   100  =  "no way jay"
  **********************************************************************/
-PRIORITY grade_sharpness(register SPLIT *split) {
+PRIORITY Wordrec::grade_sharpness(register SPLIT *split) {
   register PRIORITY grade;
 
   grade = point_priority (split->point1) + point_priority (split->point2);
@@ -172,7 +175,7 @@ PRIORITY grade_sharpness(register SPLIT *split) {
   else
     grade += 360.0;
 
-  grade *= sharpness_knob;       /* Values 0 to -360 */
+  grade *= chop_sharpness_knob;       /* Values 0 to -360 */
 
   return (grade);
 }
@@ -185,7 +188,7 @@ PRIORITY grade_sharpness(register SPLIT *split) {
  *   0    =  "perfect"
  *   100  =  "no way jay"
  **********************************************************************/
-PRIORITY grade_width_change(register BOUNDS_RECT rect) {
+PRIORITY Wordrec::grade_width_change(register BOUNDS_RECT rect) {
   register PRIORITY grade;
   register inT32 width1;
   register inT32 width2;
@@ -193,12 +196,12 @@ PRIORITY grade_width_change(register BOUNDS_RECT rect) {
   width1 = rect[3] - rect[2];
   width2 = rect[1] - rect[0];
 
-  grade = 20 - (max (rect[1], rect[3])
-    - min (rect[0], rect[2]) - max (width1, width2));
+  grade = 20 - (MAX (rect[1], rect[3])
+    - MIN (rect[0], rect[2]) - MAX (width1, width2));
 
-  grade *= width_change_knob;
+  grade *= chop_width_change_knob;
 
-  return (max (0.0, grade));
+  return (MAX (0.0, grade));
 }
 
 
@@ -207,9 +210,9 @@ PRIORITY grade_width_change(register BOUNDS_RECT rect) {
  *
  * Set up the limits for the x coordinate of the outline.
  **********************************************************************/
-void set_outline_bounds(register EDGEPT *point1,
-                        register EDGEPT *point2,
-                        BOUNDS_RECT rect) {
+void Wordrec::set_outline_bounds(register EDGEPT *point1,
+                                 register EDGEPT *point2,
+                                 BOUNDS_RECT rect) {
   register EDGEPT *this_point;
   register inT16 x_min;
   register inT16 x_max;
@@ -224,3 +227,5 @@ void set_outline_bounds(register EDGEPT *point1,
   rect[2] = x_min;
   rect[3] = x_max;
 }
+
+}  // namespace tesseract
